/*
 * Copyright 2013 Canonical Ltd.
 *
 * This file is part of ubuntu-calculator-app.
 *
 * ubuntu-calculator-app is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * ubuntu-calculator-app is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.0
import QtQuick.LocalStorage 2.0

Item {

    property var db: null

    function openDB() {
        if(db !== null) return;

        db = LocalStorage.openDatabaseSync("ubuntu-calculator-app", "", "Default Ubuntu touch calculator", 100000);

        if (db.version == "") {
            db.changeVersion("", "0.1",
                function(tx) {
                    tx.executeSql('CREATE TABLE IF NOT EXISTS Calculations(id INTEGER PRIMARY KEY, calc TEXT)');
                    console.log('Database created');
                });
        }

        if (db.version == "0.1") {
            db.changeVersion("0.1", "0.1.1",
                function(tx) {
                    tx.executeSql('ALTER TABLE Calculations ADD insertDate INTEGER NOT NULL DEFAULT 0');
                    console.log('Database upgraded to 0.1.1');
                });
        }
    }

    function getCalculations(callback){
        openDB();
        db.transaction(
                    function(tx){
                        var res = tx.executeSql('SELECT * FROM Calculations');
                        for(var i=res.rows.length - 1; i >= 0; i--){
                            var obj = JSON.parse(res.rows.item(i).calc);
                            obj.dbId = res.rows.item(i).id;
                            obj.dateText  = fromDateToString(res.rows.item(i).insertDate);

                            if (!('mainLabel' in obj))
                                obj.mainLabel = ''

                            callback(obj);
                        }
                    }
                    );
    }

    function saveCalculations(calculations){
        openDB();
        var res;
        db.transaction( function(tx){
            for(var i=0; i<calculations.length; i++){
                if(calculations[i].calc.dbId === -1) {
                    var r = tx.executeSql('INSERT INTO Calculations(calc, insertDate) VALUES(?, ?)', [JSON.stringify(calculations[i].calc), calculations[i].date]);
                    res = r.insertId;
                } else {
                    tx.executeSql('UPDATE Calculations SET calc = ? WHERE id = ?', [JSON.stringify(calculations[i].calc), calculations[i].calc.dbId]);
                    res = calculations[i].calc.dbId;
                }
            }
        });
        return res;
    }

    function removeCalculation(calc){
        openDB();
        db.transaction(function(tx){
            tx.executeSql('DELETE FROM Calculations WHERE id = ?', [calc.dbId]);
        });
    }

    /*
     * Function fromDateToString
     *
     * Given an input object recovers its timestamp and
     * returns a string based on how much time has passed
     */
    function fromDateToString(time) {
        // TODO: see in settings if user wants AM/PM or 24h
        // Now all strings are in AM/PM
        // Below every string there is a comment with string in 24h
        var then = new Date(time);
        var now = new Date();
        var daysElapsed = now.getDate() - then.getDate();
        var hoursElapsed = now.getHours() - then.getHours();
        var minutesElapsed = now.getMinutes() - then.getMinutes();
        var issameMonth = (now.getMonth() === then.getMonth());
        var relativeTime;

        // Today
        if (daysElapsed === 0 && issameMonth) {
            // Between 0 & 59 minutes
            if (hoursElapsed < 1 || (hoursElapsed === 1 && then.getMinutes() > now.getMinutes())) {
                // This is for different hours e.g. datestamp 11:58 now 12:02 prints "4 minutes ago"
                minutesElapsed = minutesElapsed > 0 ? minutesElapsed : minutesElapsed + 60;
                relativeTime = i18n.tr("%1 minute ago", "%1 minutes ago", minutesElapsed).arg(minutesElapsed);

                // Less than one minute
                if (minutesElapsed < 1 && minutesElapsed >= 0 )
                    relativeTime = i18n.tr("A few seconds ago...");

                return relativeTime;
            }

            // > 1 hour ago
            relativeTime = i18n.tr("%1 hour ago", "%1 hours ago", hoursElapsed).arg(hoursElapsed);
        }

        // Between two days and one week ago
        else if (daysElapsed > 1 && daysElapsed < 7 && issameMonth) {
            relativeTime = Qt.formatDateTime(then, "ddd, hh:mm AP");
        }

        // Yesterday
        else if (daysElapsed === 1 && issameMonth) {
            // FIXME: we need a way to not split this string
            // TRANSLATORS: What comes after 'at' is a time, e.g. 'Yesterday at 05:55 PM'
            relativeTime = i18n.tr("Yesterday at %1").arg(Qt.formatDateTime(then, "hh:mm AP"))
        }

        // More than one week ago
        else {
            relativeTime = Qt.formatDateTime(then, "MMM dd, hh:mm AP");
        }

        return relativeTime
    }

    // Function to have time from an ID
    function fromIdToDate (dbId) {
        var time;
        openDB();
        db.transaction(
            function(tx){
                var res = tx.executeSql('SELECT insertDate FROM Calculations WHERE id = ?', dbId);
                time = res.rows.item(0).insertDate;
             }
        );
        return time;
    }
}
