/*
 * Copyright 2013 Canonical Ltd.
 *
 * This file is part of ubuntu-calculator-app.
 *
 * ubuntu-calculator-app is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * ubuntu-calculator-app is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

Qt.include("engine.js");

var context = new Context;

function Formula() {
    var previous = new Token(0, T_NUMBER);
    var formula = '';
    var results = [];
    var lookupTbl = {
        '-': '−',
        '/': '÷',
        '*': '×'
    };

    var _calculate = function(func){
        try{
            var result = parse(func, context);
            if (result === Number.POSITIVE_INFINITY)
                result = '∞';
            else if (result === Number.NEGATIVE_INFINITY)
                result = '-∞';
        }catch(exception){
            if(exception instanceof DivisionByZeroError){
                result = "division by zero error";
            } else{
                throw new SyntaxError("malformed calculation");
            }
        }
        return result;
    }

    this.push = function(digit){
        if(results.length > 0) throw new SyntaxError("cannot push");
        var last = (new Scanner(previous.value.toString()+digit, context)).tokens.pop();
        switch(last.type){
        case T_PLUS:
        case T_MINUS:
        case T_DIV:
        case T_TIMES:
            if(previous.type & T_OPERATOR) return;
            break;
        case T_UNARY_PLUS:
        case T_UNARY_MINUS:
            if(previous.type === T_UNARY_MINUS || previous.type === T_UNARY_PLUS ) return;
            break;
        }
        formula += digit;
        previous = last;
        var ret = {value: '', type: last.type};
        if(lookupTbl[digit] !== undefined){
            ret.value = lookupTbl[digit];
        } else {
            ret.value = digit;
        }
        return ret;
    };

    this.pop = function(){
        if(results.length > 0){
            results.pop();
            return;
        }
        var scannedFormula = new Scanner(formula, context);
        var last = {type:T_NUMBER};
        var removeSize = 0;
        while(scannedFormula.tokens.length > 0 && (last.type === T_NUMBER || last.type === T_UNARY_PLUS || last.type === T_UNARY_MINUS)){
            last = scannedFormula.tokens.pop();
            removeSize += last.value.toString().length
        }
        formula = formula.substring(0, formula.length - removeSize);
        previous = scannedFormula.tokens.pop();
    };

    this.calculate = function(){
        var result;
        var previousSize = previous.value.toString().length;
        if(formula.length <= previousSize) return;
        if(results.length > 0){
            var scannedFormula = new Scanner(formula, context);
            var last = scannedFormula.tokens.pop();
            var newFormula = last.value.toString();
            while(last.type !== T_PLUS &&
                  last.type !== T_MINUS &&
                  last.type !== T_DIV &&
                  last.type !== T_TIMES ){
                last = scannedFormula.tokens.pop();
                newFormula = last.value.toString() + newFormula;
            }
            result = _calculate(results[results.length-1]+newFormula);
        } else {
            result = _calculate(formula);
        }
        results.push(result.toString());
        return result;
    }

    this.changeSign = function(){
        if(results.length > 0) throw new SyntaxError("cannot edit formula");
        switch(previous.type){
        case T_PLUS:
        case T_MINUS:
        case T_DIV:
        case T_TIMES:
            formulaPush("-");
            break;
        default:
            var scannedFormula = new Scanner(formula, context);
            var last = {type: undefined};
            var toAdd = [];
            var addMinus = true;
            while(scannedFormula.tokens.length > 0 && last.type !== T_PLUS &&
                  last.type !== T_MINUS &&
                  last.type !== T_DIV &&
                  last.type !== T_TIMES ){
                last = scannedFormula.tokens.pop();
                if(last.type !== T_UNARY_MINUS && last.type !== T_UNARY_PLUS){
                    toAdd.push(last);
                } else {
                    if(last.type === T_UNARY_MINUS)
                        addMinus = false;
                }
            }
            formulaPop();
            while(toAdd.length > 0){
                var el = toAdd.pop()
                formulaPush(el.value.toString());
                if((el.type & T_OPERATOR) && addMinus){
                    formulaPush('-');
                }
            }
        }
    }

    this.hasResults = function(){
        if(results.length > 0){
            return true;
        }
        return false;
    }
}
