# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2013 Canonical
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.


"""Tests for the Calculator App"""

from __future__ import absolute_import

from testtools.matchers import Equals
from autopilot.matchers import Eventually

from ubuntu_calculator_app.tests import CalculatorTestCase
from ubuntu_calculator_app.emulators.simple_page import SimplePage


class TestSimplePage(CalculatorTestCase):

    _BUTTONS_DICT = {
        "0": "zero",
        "1": "one",
        "2": "two",
        "3": "three",
        "4": "four",
        "5": "five",
        "6": "six",
        "7": "seven",
        "8": "eight",
        "9": "nine",
        "+": "plus",
        "-": "minus",
        "/": "divide",
        "*": "multiply",
        "=": "equals",
    }

    @property
    def simple_page(self):
        return SimplePage(self.app)

    def test_operation_after_clear(self):
        """ Test that after clearing one calculation, the next is still correct
        (bug #1164973).

        """
        self._calculate("1+1")

        # make sure the result is already there
        self._assert_result("2")

        # now clear the input by pressing "AC"
        self.click_button("clear")

        self._calculate("1+1")

        # make sure the result is correct
        self._assert_result("2")

    def _calculate(self, operation):
        self._click_buttons(operation + "=")

    def _click_buttons(self, input_):
        for char in input_:
            self.click_button(self._BUTTONS_DICT[char])

    def _assert_result(self, expected_result):
        get_result = lambda: self.simple_page.get_formula_label().numbers
        self.assertThat(
            get_result, Eventually(Equals(expected_result)))

    def test_equals_dont_change_numbers(self):
        """ Test that typing a number and pressing "=" won't change the number
        (bug #1165894).

        """
        self._click_buttons("123")

        # check that the label is displaying correctly
        self._assert_result("123")

        # click the "=" button
        self._click_buttons("=")

        # check that the label is still displaying correctly
        self._assert_result("123")

    def test_multiply_priority(self):
        """Multiply is done before addition or substraction."""
        self._calculate("2+2*2")
        self._assert_result("6")

    def test_divide_priority(self):
        """Divide is done before addition or substraction."""
        self._calculate("5+6/2")
        self._assert_result("8")

    def test_swipe_up_clears(self):
        """Make sure swiping up clears the formulae label."""
        screen = self.simple_page.get_screen()
        self._click_buttons("9")

        x, y, h, w = screen.globalRect
        tx = x + (h / 2)
        ty = y + (w / 2)

        self.pointing_device.drag(tx, ty + w, tx, ty - w)

        self._assert_result("")

    def test_sign_button(self):
        """Ensure the sign button is working."""
        self._click_buttons("5+")
        self.click_button("sign")
        self._click_buttons("1=")

        self._assert_result("4")

    def test_addition(self):
        self._calculate("4+2")
        self._assert_result("6")

    def test_substraction(self):
        self._calculate("5-4")
        self._assert_result("1")

    def test_positive_numbers_multiplication(self):
        self._calculate("2*3")
        self._assert_result("6")

    def test_multiplication_by_zero(self):
        self._calculate("4*0")
        self._assert_result("0")

    def test_one_negative_number_multiplication(self):
        self._calculate("-3*4")
        self._assert_result("-12")

    def test_two_negative_numbers_multiplication(self):
        self._calculate("-2*-3")
        self._assert_result("6")

    def test_three_positive_numbers_multiplication(self):
        self._calculate("2*3*10")
        self._assert_result("60")

    def test_two_negatives_in_three_numbers_multiplication(self):
        self._calculate("-2*-3*5")
        self._assert_result("30")

    def test_three_negative_numbers_multiplication(self):
        self._calculate("-4*-5*-7")
        self._assert_result("-140")

    def test_divide_positive(self):
        self._calculate("4/2")
        self._assert_result("2")

    def test_divide_negative(self):
        self._calculate("4/-2")
        self._assert_result("-2")

    def test_divide_two_negatives(self):
        self._calculate("-4/-2")
        self._assert_result("2")

    def test_divide_with_zero(self):
        self._calculate("0/4")
        self._assert_result("0")

    def test_divide_by_zero(self):
        self._calculate("4/0")
        self._assert_result(u'\u221e') # Unicode for "infinite" result
